<?php

namespace Drupal\monarch_search_block\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a customizable search form.
 */
class SearchForm extends FormBase {

  /**
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   */
  public function __construct(ConfigFactoryInterface $config) {
    $this->config = $config;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'monarch_search_block_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#attached']['library'][] = 'monarch_search_block/monarch_search_block';
    $form['#attributes']['class'][] = 'monarch-search-block-form';

    $services = $this->config('monarch_search_block.settings')->get('services');

    // Order by weight
    usort($services, function ($a, $b) {
      return $a['weight'] <=> $b['weight'];
    });

    $form['search_service'] = [
      '#type' => 'select',
      '#title' => $this->t('Search Option'),
      '#title_display' => 'invisible',
    ];
    foreach ($services as $delta => $service) {
      $form['search_service']['#options'][$delta] = $service['title'];
    }

    $form['search_query'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search Keywords'),
      '#title_display' => 'invisible',
      '#required' => TRUE,
      '#attributes' => [
        'placeholder' => $this->config('monarch_search_block.settings')->get('placeholder'),
      ],
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Search'),
      '#button_type' => 'primary',
    ];

    if ($this->config('monarch_search_block.settings')->get('new_tab')) {
      $form['actions']['submit']['#attributes']['formtarget'] = '_blank';
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $services = $this->config('monarch_search_block.settings')->get('services');

    // Order by weight
    usort($services, function ($a, $b) {
      return $a['weight'] <=> $b['weight'];
    });

    $delta = $form_state->getValue('search_service');
    $selected_service = $services[$delta];

    $redirect_url = $selected_service['base_url'] . '?' . $selected_service['param'] . '=' . $form_state->getValue('search_query');
    $response = new TrustedRedirectResponse($redirect_url);
    $metadata = $response->getCacheableMetadata();
    $metadata->setCacheMaxAge(0);
    $form_state->setResponse($response);
  }

}
